﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Barcodes;
using System.Windows.Media.Imaging;
using System.IO;

namespace barcode
{
    public partial class MainPage : UserControl
    {
        BitmapImage image = new BitmapImage(new Uri("Mixed_Ean_QR_codes.png", UriKind.RelativeOrAbsolute));
        BarcodeReader barcodeReader = new BarcodeReader();
        CaptureSource captureSource;
        public MainPage()
        {
            InitializeComponent();

            barcodeReader.DecodeComplete += new EventHandler<ReaderDecodeArgs>(barcodeReader_DecodeComplete);
            //assign initial image
            ImageContainer.Source = image;
            //preselect symbology
            SymbologyCombo.SelectedIndex = 0;
        }

        //Completed Decode - use the result value and optionally show filtered image
        //This event can get fired multiple times for one Decode operation
        void barcodeReader_DecodeComplete(object sender, ReaderDecodeArgs e)
        {
            FilteredImageContainer.Source = e.FilteredImage;
            if (e.SymbolFound)
            {
                resultText.Text += "\n Symbology: " + e.Symbology + "; Data: " + e.Value;
            }
            else
            {
                resultText.Text = "Failed attempt.";
            }
        }

        /// <summary>
        /// Start the barcode reader's decode method with some settings.
        /// </summary>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // Only read up to two barcode found on the image (usually the first two from left to right)
            barcodeReader.MaxNumberOfSymbolsToRead = 2;
            BitmapSource bs = (BitmapSource)ImageContainer.Source;
            // Decode the desired source using the symbology set in the combo!
            // It's a massive time saver when symbology is specified(other than 'All')!
            barcodeReader.Decode(bs, (Symbology)this.SymbologyCombo.SelectionBoxItem); 
        }

        /// <summary>
        /// Open file dialog for the user to pick an image and set it to the container.
        /// </summary>
        private void Open_Local_Image(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Multiselect = false;
            dlg.Filter = "Images (*.png, *.jpg, *.bmp)|*.png;*.jpg; *.bmp";
            var result = dlg.ShowDialog();
            if (result == true)
            {
                var something = dlg.File.OpenRead();
                BitmapImage uploaded = new BitmapImage();
                uploaded.SetSource(something);
                ImageContainer.Source = uploaded;
            }
        }

        /// <summary>
        /// Initiate new service client to download the image with URL entered by the user.
        /// </summary>
        private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            try
            {
                ServiceReference1.WorkaroundServiceClient client = new ServiceReference1.WorkaroundServiceClient();
                client.DoWorkCompleted += new EventHandler<ServiceReference1.DoWorkCompletedEventArgs>(client_DoWorkCompleted);
                client.DoWorkAsync((sender as TextBox).Text);
            }
            catch (Exception ex)
            {
                (sender as TextBox).Text = ex.Message;
            }
        }

        void client_DoWorkCompleted(object sender, ServiceReference1.DoWorkCompletedEventArgs e)
        {
            if (e.Result != null)
            {
                try
                {
                    //transform the result to image and set it to the container
                    byte[] ImageBytes = e.Result;
                    MemoryStream ms = new MemoryStream(e.Result);
                    BitmapImage bmp = new BitmapImage();
                    bmp.SetSource(ms);
                    ImageContainer.Source = bmp;
                }
                catch (InvalidOperationException ex)
                {
                    MessageBox.Show("Something went terribly wrong.... Here, have a look: " + ex.Message);
                }
            }
            else MessageBox.Show("Failed to retrieve that image. Please check your link and try again");
        }

        /// <summary>
        /// Set up the capture source, the video brush and access. Then start the cam.
        /// </summary>
        private void StartCam(object sender, RoutedEventArgs e)
        {

            captureSource = new CaptureSource();
            captureSource.VideoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice(); //null when no cam is available!
            //setup video fill
            VideoBrush webcamBrush = new VideoBrush();
            webcamBrush.SetSource(captureSource);
            webcamBrush.Stretch = Stretch.Uniform;
            Video.Fill = webcamBrush;
            //if device is available and access is granted
            if (CaptureDeviceConfiguration.RequestDeviceAccess() && captureSource.VideoCaptureDevice != null)
            {
                try
                {
                    //start the cam
                    captureSource.Start();
                    Start.IsEnabled = false;
                    Stop.IsEnabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Something went terribly wrong.... Here, have a look: " + ex.Message);
                }
            }
            else MessageBox.Show("It appears you don't have a working WebCam. ..Right?");
        }

        /// <summary>
        /// Initiate a capture request and stop the cam once done.
        /// </summary>
        private void StopCam(object sender, RoutedEventArgs e)
        {
            if (captureSource.VideoCaptureDevice != null && captureSource.State == CaptureState.Started)
            {
                captureSource.CaptureImageCompleted += new EventHandler<CaptureImageCompletedEventArgs>(source_CaptureImageCompleted);
                captureSource.CaptureImageAsync();
            }
        }

        void source_CaptureImageCompleted(object sender, CaptureImageCompletedEventArgs e)
        {
            if (captureSource.VideoCaptureDevice != null)
            {
                //stop the cam
                captureSource.Stop();
                //set the result in the image container
                ImageContainer.Source = e.Result;
                Start.IsEnabled = true;
                Stop.IsEnabled = false;
            }
        }
    }
}
